//---------------------------------------------------------------------------
//
//   ObjectViewSwitcher
//
//          Copyright(C) 2013, Tark
//
//  IuWFNgpl̃IuWFNgXg
//
//---------------------------------------------------------------------------

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <memory>
#include <sstream>
#include <algorithm>

#include "ObjectViewSwitcherWindow.h"
#include "ObjectViewSwitcher.h"
#include "MQSetting.h"

static ObjectViewSwitcher plugin;

// Constructor
// RXgN^
ObjectViewSwitcher::ObjectViewSwitcher() : m_Window(nullptr), m_bActivate(false), m_storagesize(5), m_newstoragesize(m_storagesize)
{
	setlocale(LC_CTYPE, "");
}

// Destructor
// fXgN^
ObjectViewSwitcher::~ObjectViewSwitcher()
{
	if(m_Window != NULL){
		delete m_Window;
		m_Window = NULL;
	}
}

//---------------------------------------------------------------------------
//  GetPlugInID
//---------------------------------------------------------------------------
void ObjectViewSwitcher::GetPlugInID(DWORD *Product, DWORD *ID)
{
	*Product = 0xE3F7A830;
	*ID      = 0xBD9A1874;
}

//---------------------------------------------------------------------------
//  GetPlugInName
//---------------------------------------------------------------------------
const char *ObjectViewSwitcher::GetPlugInName(void)
{
	return "ObjectViewSwitcher v.1.2  Copyright(C) 2014, Tark.";
}

//---------------------------------------------------------------------------
//  EnumString
//---------------------------------------------------------------------------
const char *ObjectViewSwitcher::EnumString(void)
{
	return "ObjectViewSwitcher";
}

//---------------------------------------------------------------------------
//  Initialize
//---------------------------------------------------------------------------
BOOL ObjectViewSwitcher::Initialize()
{
	int posX = -1, posY = -1;
	{
		MQSetting *setting = OpenSetting();
		setting->Load("ObjectViewSwitcher_activate", m_bActivate, false);
		setting->Load("PosX", posX, -1);
		setting->Load("PosY", posY, -1);
		CloseSetting(setting);
	}
	if (m_Window == NULL){
		MQWindowBase mainwnd = MQWindow::GetMainWindow();
		m_Window = new ObjectViewSwitcherWindow(this, mainwnd);
		if(posX >= 0 && posY >= 0) {
			m_Window->SetPosX(posX);
			m_Window->SetPosY(posY);
		}
	}
	Activate(nullptr, m_bActivate);
	return TRUE;
}

//---------------------------------------------------------------------------
//  Exit
//    AvP[V̏I
//---------------------------------------------------------------------------
void ObjectViewSwitcher::Exit()
{
	{
		MQSetting *setting = OpenSetting();
		setting->Save("ObjectViewSwitcher_activate", m_bActivate);
		setting->Save("PosX", m_Window->GetPosX());
		setting->Save("PosY", m_Window->GetPosY());
		CloseSetting(setting);
	}
	if (m_Window != NULL){
		delete m_Window;
		m_Window = NULL;
	}
}

//---------------------------------------------------------------------------
//  Activate
//---------------------------------------------------------------------------
BOOL ObjectViewSwitcher::Activate(MQDocument doc, BOOL flag)
{
//	m_doc = doc;
	if (m_Window == NULL) {
		return FALSE;
	}
	m_bActivate = flag ? true : false;
	m_Window->SetVisible(m_bActivate);
	m_Window->resize();

	return m_bActivate;
}

//---------------------------------------------------------------------------
//  IsActivated
//---------------------------------------------------------------------------
BOOL ObjectViewSwitcher::IsActivated(MQDocument doc)
{
	if (m_Window == NULL) {
		return FALSE;
	}
	return m_bActivate;
}

//---------------------------------------------------------------------------
//  ExecuteCallback
///---------------------------------------------------------------------------
bool ObjectViewSwitcher::ExecuteCallback(MQDocument doc, void *option)
{
	CallbackInfo *info = (CallbackInfo*)option;
	return ((*this).*info->proc)(doc);
}

// R[obŇĂяo
void ObjectViewSwitcher::Execute(ExecuteCallbackProc proc)
{
	CallbackInfo info;
	info.proc = proc;
	BeginCallback(&info);
}

// vOC̃x[XNXԂ
MQBasePlugin *GetPluginClass()
{
	return &plugin;
}

// Zbg
BOOL ObjectViewSwitcher::set_list(MQDocument doc_, int index, bool selectedonly)
{
	cmq::CMQDocument doc(doc_);
	if(m_viewlist.size() <= index) {
		m_viewlist.resize(m_storagesize);
	}
	auto &viewmap = m_viewlist[index];
	viewmap.clear();
	int ocount = doc.getObjectCount();
	for(int oidx = 0; oidx < ocount; oidx++) {
		cmq::CMQObject obj(doc.getObject(oidx));
		if(!obj.isLive()) {
			continue;
		}
		if(selectedonly && !obj.getSelected()) {
			continue;
		}
		std::string name = obj.getName();
		bool view = obj.getVisible();
		auto it = viewmap.find(name);
		if(it != viewmap.end()) {
			it->second = view;
		}
		else {
			viewmap.insert(std::make_pair(name, view));
		}
	}
	return FALSE;
}

// XgA
BOOL ObjectViewSwitcher::restore_list(MQDocument doc_, int index)
{
	cmq::CMQDocument doc(doc_);
	if(m_viewlist.size() <= index || m_viewlist[index].empty()) {
		return FALSE;
	}
	auto &viewmap = m_viewlist[index];
	int ocount = doc.getObjectCount();
	for(int oidx = 0; oidx < ocount; oidx++) {
		cmq::CMQObject obj(doc.getObject(oidx));
		if(!obj.isLive()) {
			continue;
		}
		std::string name = obj.getName();
		auto findit = viewmap.find(name);
		if(findit != viewmap.end()) {
			obj.setVisible(findit->second);
		}
	}
	return TRUE;
}

// NA
BOOL ObjectViewSwitcher::clear_list(MQDocument doc_, int index)
{
	cmq::CMQDocument doc(doc_);
	if(m_viewlist.size() <= index || m_viewlist[index].empty()) {
		return FALSE;
	}
	auto &viewmap = m_viewlist[index];
	viewmap.clear();
	return FALSE;
}

bool ObjectViewSwitcher::hasdata(int index) const
{
	return !(m_viewlist.size() <= index || m_viewlist[index].empty());
}

// ǂݍ
void ObjectViewSwitcher::OnNewDocument(MQDocument doc, const char *filename, NEW_DOCUMENT_PARAM& param)
{
	MQXmlElement root = param.elem->FirstChildElement();
	MQXmlElement list = root->FirstChildElement();
	m_Window->clearbutton();
	m_viewlist.clear();
	m_storagesize = 5;
	int index = 0;
	while (list){
		MQXmlElement item = list->FirstChildElement();
		std::unordered_map<std::string, bool> viewlist;
		while(item) {
			std::string name = item->GetText();
			bool visible = (item->GetAttribute("visible") == "true");
			viewlist.insert(std::make_pair(name, visible));
			item = list->NextChildElement(item);
		}
		m_viewlist.push_back(viewlist);
//		if(!viewlist.empty()) {
//			m_Window->setbutton(index, true);
//		}
		index++;
		list = root->NextChildElement(list);
	}
	if(m_storagesize < m_viewlist.size()) {
		m_storagesize = m_viewlist.size();
	}
	if(m_Window) {
		m_Window->resizeButtonList();
	}
}

// ۑ
void ObjectViewSwitcher::OnSaveDocument(MQDocument doc, const char *filename, SAVE_DOCUMENT_PARAM& param)
{
	MQXmlElement root = param.elem->AddChildElement("Store");
	auto findit = std::find_if(m_viewlist.rbegin(), m_viewlist.rend(), [](std::unordered_map<std::string, bool> &viewlist) { return !viewlist.empty(); });
	if(findit == m_viewlist.rend()) {
		m_viewlist.clear();
	}
	else {
		size_t size = &(*findit) - &m_viewlist.front() + 1;
		m_viewlist.resize(size);
	}
	for(auto view : m_viewlist) {
		MQXmlElement list = root->AddChildElement("list");
		for(auto visible : view) {
			MQXmlElement item = list->AddChildElement("name");
			item->SetText(visible.first.c_str());
			item->SetAttribute("visible", visible.second ? "true" : "false");
		}
	}
}

BOOL ObjectViewSwitcher::addSlot()
{
	m_storagesize++;
	return FALSE;
}
